// Copyright (c) 2020 FOOSYS. All rights reserved.
#pragma once

#include "LibFunc.hpp"
#include "ResourceManager.hpp"

namespace FOOSYS
{

#define AnException(messageID) AnExceptionImpl(messageID, __FILE__, __LINE__, ::GetLastError());

static VOID formatterA(std::stringstream& ss)
{
	ss << "\r\n";
}

template <class Thead, class... Tbody>
VOID formatterA(std::stringstream& ss, Thead&& head, Tbody&&... body)
{
	ss << std::forward<Thead>(head);
	if (sizeof...(body) > 0)
		formatterA(ss);
	formatterA(ss, std::forward<Tbody>(body)...);
}

template <class... Types>
std::string makeMessageA(Types&&... args)
{
	std::stringstream ss;
	formatterA(ss, std::forward<Types>(args)...);
	return std::string(ss.str());
}

class AnExceptionImpl final : public std::runtime_error
{
public:
	AnExceptionImpl(ResourceManager::ID stringEnum, const CHAR* file, INT line, DWORD winErrorCode)
		: std::runtime_error(
			makeMessageA(
				ResourceManager::get().getStringA(ResourceManager::ID::CaptureFailed),
				ResourceManager::get().getStringA(stringEnum),
				"", // s
				"File=" + nfl::getFileNameFromFullPathA(file), 
				"Line=" + std::to_string(line), 
				"GetLastError()=" + std::to_string(winErrorCode)
			))
	{}
	AnExceptionImpl() = delete;
};

} // namespace FOOSYS
