// Copyright (c) 2020 FOOSYS. All rights reserved.
#pragma once

#include "ButtonActor.hpp"
#include "Desktop.hpp"
#include "MessagePump.hpp"
#include "CaptureFilePathCreator.hpp"

namespace FOOSYS
{

#define DECLARE_MESSAGE_PUMP \
	static MessagePump messagePump_; \
	static LRESULT CALLBACK wndProc(HWND hWnd, UINT messageID, WPARAM wParam, LPARAM lParam) \
	{ \
		return messagePump_.wndProc(hWnd, messageID, wParam, lParam); \
	}

#define CREATE_WINDOW __super::createWindow(wndProc, messagePump_)

class IniFile;

class CaptureBase abstract : public ButtonActor
{

public:
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// RXgN^AfXgN^
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	CaptureBase(HWND hWndParent, const std::wstring& uniqueIdentifier);
	CaptureBase() = delete;
	virtual ~CaptureBase() = 0 {};

protected:
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// z֐
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	virtual VOID onCreate();
	virtual VOID onShowFirst();
	virtual VOID onLButtonDown(POINT&& point);
	virtual VOID onLButtonUp(POINT&& point);
	virtual VOID onRButtonUp(POINT&& point);
	virtual VOID onMouseMove(POINT&& point);
	virtual bool onKeyDown(UINT virtualKeyCode, UINT option);

	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// wp[֐
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
protected:
	VOID createWindow(WNDPROC wndProc, MessagePump& messagePump);
	VOID doCapture() const;
	//VOID doCapture(const RECT& rect) const;
	VOID doCapture(const POINT& point, const SIZE& size) const;
	VOID captureDesktop();
	VOID hideCanvas() const;
	VOID showCapture() const;
	VOID showCapture(const RECT& rect) const;
	VOID showCapture(const POINT& point, const SIZE& size) const;
	VOID showCanvas() const;
	VOID showMouseToolTip(const POINT& point, std::wstring const& mouseToolTip) const;
	VOID refreshCanvas() const;
	VOID clearCanvas() const;
	VOID mergeFrame(HDC hDcSrc, const RECT& rect) const;
	RECT getDesktopRect() const;

	VOID completionNotify() const;
	VOID drawCursor() const;
	VOID drawRectangleCaptureFrame(const POINT& point1, const POINT& point2) const;
	VOID drawMagnifier(const POINT& point) const;
	VOID drawFrame(const RECT& rect) const;
	VOID drawAuxiliaryLine(const POINT& point) const;
	VOID drawAuxiliaryCircle(const POINT& point) const;
	RECT&& clipRect(RECT&& rect) const;
	bool cancel();

private:
	VOID registerMessageHandler(MessagePump& messagePump);

	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// C^[tF[X̎֐
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
private:
	VOID doIt(nfl::FVV_t completionNotice) override;


	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// bZ[Wnh
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
private:
	VOID onLButtonDownBase(WPARAM wParam, LPARAM lParam);
	VOID onLButtonUpBase(WPARAM wParam, LPARAM lParam);
	VOID onRButtonUpBase(WPARAM wParam, LPARAM lParam);
	VOID onMouseMoveBase(WPARAM wParam, LPARAM lParam);
	VOID onSetFocusBase();
	VOID onKillFocusBase();
	VOID onPaintBase();
	VOID onKeyDownBase(WPARAM wParam, LPARAM lParam);

	///////////////////////////////////////////////////////////////////////////////////////////////////////////
	// o[ϐ
	///////////////////////////////////////////////////////////////////////////////////////////////////////////
protected:
	HWND hWndParent_;
	bool isCapturing_;
	bool isCaptureWithKey_;

private:
	HWND hWndCanvas_;
	nfl::FVV_t completionNotice_;
	std::wstring uniqueIdentifier_;
	std::unique_ptr<Desktop> desktop_;
	CaptureFilePathCreator captureFilePathCreator_;

};

} // namespace FOOSYS
