// Copyright (c) 2020 FOOSYS. All rights reserved.
#include "stdafx.h"
#include "GeneralSettings.hpp"
#include "resource.h"
#include "LibFunc.hpp"
#include "GdiHelper.hpp"

namespace FOOSYS
{

HWND GeneralSettings::hDlgThis_;
MessagePump GeneralSettings::messagePump_;

GeneralSettings::GeneralSettings(ResourceManager::ID id, const RECT& area)
: SettingsBase(id, area), hUrl_(nullptr)
{
	messagePump_.addHandler(std::make_pair(WM_INITDIALOG, std::bind(&GeneralSettings::onInit, this, std::placeholders::_1, std::placeholders::_2)));
	messagePump_.addHandler(std::make_pair(WM_LBUTTONUP, std::bind(&GeneralSettings::onLButtonUp, this, std::placeholders::_1, std::placeholders::_2)));
	messagePump_.addHandler(std::make_pair(WM_MOUSEMOVE, std::bind(&GeneralSettings::onMouseMove, this, std::placeholders::_1, std::placeholders::_2)));
	messagePump_.addReturnValueHandler(std::make_pair(WM_CTLCOLORSTATIC, std::bind(&GeneralSettings::onCtlColorStatic, this, std::placeholders::_1, std::placeholders::_2)));
}

VOID GeneralSettings::create(HWND hWndParent)
{
	::CreateDialog(resourceManager_.getHInstance(), MAKEINTRESOURCE(IDD_GENERALSETTINGSDLG), hWndParent, dlgProc);
}

VOID GeneralSettings::onInit(WPARAM wParam, LPARAM lParam)
{
	__super::setChildDlgHandle(hDlgThis_);
	__super::move();
	nfl::setWindowText(hDlgThis_, IDC_STA_VERSION, (resourceManager_.getAppName() + resourceManager_.getString(ResourceManager::ID::SettingsDlgVersion)).c_str());
	nfl::setWindowText(hDlgThis_, IDC_STA_COPYRIGHT, resourceManager_.getStringPointer(ResourceManager::ID::SettingsDlgCopyright));
	nfl::setWindowText(hDlgThis_, IDC_STA_LINK, resourceManager_.getStringPointer(ResourceManager::ID::SettingsDlgHomePage));
	hUrl_ = ::GetDlgItem(hDlgThis_, IDC_STA_LINK);
}

VOID GeneralSettings::onLButtonUp(WPARAM wParam, LPARAM lParam)
{
	POINT point = nfl::getScreenCoordinateFromLParam(hDlgThis_, lParam);
	RECT rectUrl = nfl::getObjectRect_f(hUrl_, ::GetWindowRect);
	if (::PtInRect(&rectUrl, point))
		showLink();
}

VOID GeneralSettings::onMouseMove(WPARAM wParam, LPARAM lParam)
{
	POINT point = nfl::getScreenCoordinateFromLParam(hDlgThis_, lParam);
	RECT rectUrl = nfl::getObjectRect_f(hUrl_, ::GetWindowRect);
	if (::PtInRect(&rectUrl, point))
	{
		auto hCursorHand = ::LoadCursor(NULL, IDC_HAND);
		::SetCursor(hCursorHand);
	}
}

INT_PTR GeneralSettings::onCtlColorStatic(WPARAM wParam, LPARAM lParam)
{
	HDC hDC = reinterpret_cast<HDC>(wParam);
	HWND hCtl = reinterpret_cast<HWND>(lParam);
	if (hCtl == ::GetDlgItem(hDlgThis_, IDC_STA_LINK))
	{
		::SetBkMode(hDC, TRANSPARENT);
		::SetTextColor(hDC, GdiHelper::ColorEnum::Blue);
		return reinterpret_cast<INT_PTR>(::GetStockObject(NULL_BRUSH));
	}
	return static_cast<INT_PTR>(FALSE);
}

VOID GeneralSettings::showLink()
{
	auto url = nfl::getWindowText(hDlgThis_, IDC_STA_LINK);
	if (!nfl::launchExternalApp(hDlgThis_, url.c_str(), nullptr))
		::MessageBox(hDlgThis_, resourceManager_.getStringPointer(ResourceManager::ID::BrowserStartupFailed), resourceManager_.getAppName().c_str(), MB_OK | MB_ICONWARNING);
}

INT_PTR CALLBACK GeneralSettings::dlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	hDlgThis_ = hDlg;
	return messagePump_.dlgProc(hDlg, message, wParam, lParam);
}

} // namespace FOOSYS
